Class Image

	Private m_Width
	Public Property Get Width
		Width = m_Width
	End Property

	Private m_Height
	Public Property Get Height
		Height = m_Height
	End Property

	Private m_Type
	Public Property Get [Type]
		[Type] = m_Type
	End Property
	
	Private m_Bytes
	Public Property Get Bytes
		Bytes = m_Bytes
	End Property
	
	Public Property Get FileSize
		FileSize = LenB(m_Bytes)
	End Property

	Public Property Get Size
		If Bytes>0 Then Size = LenB(Bytes) Else Size = -1
	End Property

	Private Sub Class_Initialize : Clear : End Sub
	Private Sub Clear
		m_Bytes = -1
		m_Width = -1
		m_Height = -1
		m_Type = "NONE"
		m_Bytes = -1
	End Sub


	Public Function LoadFromBytes(bytes)
		LoadFromBytes = False : Clear
		On Error Resume Next

		m_Bytes = bytes

		'determine file type
		If MidB(Bytes,1,3) = str2bin("GIF")  Then 'GIF89aWWHH
				m_Width = bin2lng(MidB(Bytes,7,2),true)
				m_Height = bin2lng(MidB(Bytes,9,2),true)
				m_Type = "GIF"
		ElseIf MidB(Bytes,1,2) = str2bin("BM")  Then 'BMxxxxxxxxxxxxSSSSWWWW
				m_Width = bin2lng(MidB(Bytes, 19, 2),true)
				m_Height = bin2lng(MidB(Bytes, 23, 2),true)
				m_Type = "BMP"
		ElseIf MidB(Bytes,2,3) = str2bin("PNG") Then 'xPNGxxxxWWWWHHHH
			m_Width = bin2lng(MidB(Bytes, 19, 2),false)
			m_Height =  bin2lng(MidB(Bytes, 23, 2),false)
			m_Type = "PNG"
		Else
			'JPEG is a complete pain to test for. 
			If Not TrySetJPEG Then
				Clear
				Exit Function
			End If
		End If
		
		LoadFromBytes = (Err.Number = 0)
	End Function

	Private Function TrySetJPEG()
		'Source: http://www.funducode.com/freec/Fileformats/format3/format3b.htm
		TrySetJPEG = False

		'JPEG Segment := FFSSLL, SS = Type, LL = Length
		Dim curPos, segType, segSize : curPos = 1

		'start at the SOI (FFD8)
		curPos = InStrB( Bytes, ChrB(&HFF) & ChrB(&HD8) )
		If curPos = 0 Then Exit Function

		'advance to the first segment after SOI (FFxx)
		curPos = InStrB( curPos + 2, Bytes, ChrB(&HFF) )
		If curPos = 0 Then Exit Function

		'loop thru all segments in the JPEG
		Do 
			'seg type is second byte in segment
			segType = MidB(Bytes, curPos + 1 , 1)

			'test if it is the SOF Segment (FFC{0|1|2|3|5|6|7|9|A|B|D|E|F})
			If (segType >= ChrB(&HC0)) And (segType <= ChrB(&HCF)) Then
				If (segType <> ChrB(&HC4)) And (segType <> ChrB(&HC8)) And (segType <> ChrB(&HCC)) Then 
					Exit Do
				End If
			End If

			'size is bytes 3-4
			segSize = bin2lng( MidB(Bytes, curPos+2, 2) , False)

			'advance to the next
			curPos = curPos + segSize + 2
		Loop
		If curPos = 0 Then Exit Function

		'SOF := FFCxPHHWW
		m_Height = bin2lng(MidB(Bytes, curPos + 5, 2),false)
		m_Width = bin2lng(MidB(Bytes, curPos + 7, 2),false)
		m_Type = "JPG"

		TrySetJPEG = True
	End Function

	Private Function str2bin(str)
		'VBS Strings are two bytes (FF) per char, so for each char, extract the
		'second byte out
		Dim bin,i
		For i = 1 To LenB(str) Step 2 : bin = bin & MidB(str,i,1) : Next
		str2bin = bin
	End Function

	Private Function bin2lng(bin,useBigEndian)
		'Converts a "bin" type (single byte string "FF") to long integer 
		Dim lft, rgt
		If useBigEndian Then
			lft = AscB(LeftB(bin,1)) : rgt = (AscB(RightB(bin,1)) * 256)
		Else
			lft = AscB(RightB(bin,1)) : rgt = (AscB(LeftB(bin,1)) * 256)
		End If

		bin2lng = clng(lft + rgt)
	End Function

End Class

